# Generated by the protocol buffer compiler (protoc-perl) DO NOT EDIT!
# source: inline text

package MyPackage::Name::NotUsed;

use strict;
use warnings;

use Google::ProtocolBuffers;
{
    unless (My::Test::R->can('_pb_fields_list')) {
        Google::ProtocolBuffers->create_message(
            'My::Test::R',
            [
                [
                    Google::ProtocolBuffers::Constants::LABEL_OPTIONAL(), 
                    Google::ProtocolBuffers::Constants::TYPE_INT32(), 
                    'a', 1, undef
                ],
                [
                    Google::ProtocolBuffers::Constants::LABEL_OPTIONAL(), 
                    Google::ProtocolBuffers::Constants::TYPE_INT32(), 
                    'b', 10, undef
                ],
                [
                    Google::ProtocolBuffers::Constants::LABEL_OPTIONAL(), 
                    Google::ProtocolBuffers::Constants::TYPE_INT32(), 
                    'c', 11, undef
                ],

            ],
            { 'create_accessors' => 1,  }
        );
    }

    unless (My::Test::RFoo->can('_pb_fields_list')) {
        Google::ProtocolBuffers->create_message(
            'My::Test::RFoo',
            [

            ],
            { 'create_accessors' => 1,  }
        );
    }

}
1;
